/**
 * Copyright © 2014-2019 Darwino, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.darwino.domino.napi;

import com.ibm.commons.util.StringUtil;


/**
 * An exception representing a problem during LotusScript compilation.
 * 
 * @author Jesse Gallagher
 * @since 2.5.0
 */
public class LotusScriptCompilationException extends DominoException {

	private static final long serialVersionUID = 1L;

	private final int version;
	private final int line;
	private final String errorText;
	private final String errorFile;
	
    public LotusScriptCompilationException(Throwable nextException, int status, int version, int line, String errorText, String errorFile, String msg, Object...params) {
        super(nextException, status, msg, params);
        this.version = version;
        this.line = line;
        this.errorText = errorText;
        this.errorFile = errorFile;
    }
    
    public int getVersion() {
		return version;
	}
    
    public int getLine() {
		return line;
	}
    
    public String getErrorText() {
		return errorText;
	}
    
    public String getErrorFile() {
		return errorFile;
	}
    
    @Override
    public String getMessage() {
    	return StringUtil.format("{0}: [Error={1}, Line={2}, File={3}}] {7}", super.getMessage(), errorText, line, errorFile); //$NON-NLS-1$
    }
    @Override
    public String getLocalizedMessage() {
    	return getMessage();
    }
}
