/**
 * Copyright © 2014-2019 Darwino, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.darwino.domino.napi.struct;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.ibm.commons.util.QuickSort;


/**
 * Base Object for tracing allocation.
 * 
 * @author priand
 */
public abstract class NativeAllocationTracer {

	public static boolean TRACE_OBJECTS; 
	
	private static Map<Class<? extends NativeAllocationTracer>,Integer> allocated = new HashMap<Class<? extends NativeAllocationTracer>, Integer>();
	
	@SuppressWarnings("unchecked")
	public static void dump() {
		synchronized (allocated) {
			Set<Class<? extends NativeAllocationTracer>> keys = allocated.keySet();
			Class<? extends NativeAllocationTracer>[] cl = keys.toArray(new Class[keys.size()]);
			new QuickSort.ObjectArray(cl) {
				@Override
				public int compare(Object paramObject1, Object paramObject2) {
					String s1 = ((Class<?>)paramObject1).getName();
					String s2 = ((Class<?>)paramObject2).getName();
					return s1.compareTo(s2);
				}
			}.sort();;
			System.out.println("------------------------------------------------");
			for(int i=0; i<cl.length; i++) {
				System.out.println(cl[i].getName()+": "+allocated.get(cl[i]));
			}
		}
		
	}

	protected void _inc() {
		if(TRACE_OBJECTS) {
			synchronized (allocated) {
				Integer c = allocated.get(getClass());
				if(c==null) {
					c = 0;
				}
				allocated.put(getClass(), c+1);
			}
		}
	}
	protected void _dec() {
		if(TRACE_OBJECTS) {
			synchronized (allocated) {
				Integer c = allocated.get(getClass());
				if(c==null) c = 1;
				allocated.put(getClass(), c-1);
			}
		}
	}
	
	protected NativeAllocationTracer() {
	}
	
}
